package main

import (
        "context"
        "fmt"
        "log"
        "net/http"
        "os"
        "sync"
        "time"

        "github.com/cloudflare/cloudflare-go"
)

var (
        requestCount int
        startTime    time.Time
        mutex        sync.Mutex
)

const (
        threshold  = 100      //10秒内如果超过100次404，则认为当前遇到了404攻击
        timeWindow = 10 * time.Second
        attackTime = 5 * time.Minute       //开启5分钟的5秒盾
        apiKey     = "11111111"   //CF的API KEY
        email      = "a@b.com"  //CF的登录email
        zoneID     = "123"  //ZONEID
        apiURL     = "https://api.cloudflare.com/client/v4/zones/"
)

func init() {
        startTime = time.Now()
}

func handleRequest(w http.ResponseWriter, r *http.Request) {
        mutex.Lock()
        defer mutex.Unlock()

        if time.Since(startTime) > timeWindow {
                startTime = time.Now()
                requestCount = 0
        }

        requestCount++
        if requestCount >= threshold {
                setUnderAttackMode()
                requestCount = 0
                startTime = time.Now()
        }

        log.Printf("Request handled %d : %d\n", requestCount, threshold)
}

func setUnderAttackMode() {
        api, err := cloudflare.New(apiKey, email)
        if err != nil {
                log.Printf("Error creating Cloudflare API instance: %v\n", err)
        }

        // Set the Under Attack mode (example)
        // Cloudflare's Go SDK does not directly support security level changes
        // You need to interact with the API directly or through custom methods

        err = setSecurityLevel(api, zoneID, "under_attack")
        if err != nil {
                log.Printf("Error setting 'Under Attack' mode: %v\n", err)
                return
        }

        log.Printf("Cloudflare 'Under Attack' mode enabled\n")
        time.AfterFunc(attackTime, unsetUnderAttackMode)
}

func unsetUnderAttackMode() {
        api, err := cloudflare.New(apiKey, email)
        if err != nil {
                log.Printf("Error creating Cloudflare API instance: %v\n", err)
        }

        err = setSecurityLevel(api, zoneID, "medium")
        if err != nil {
                log.Printf("Error unsetting 'Under Attack' mode: %v\n", err)
                return
        }

        log.Printf("Cloudflare 'Under Attack' mode disabled\n")
}

func setSecurityLevel(api *cloudflare.API, zoneID, level string) error {
        // 创建上下文
        ctx := context.Background()

        // 定义安全级别的设置
        settings := []cloudflare.ZoneSetting{
                {
                        ID:    "security_level",
                        Value: level,
                },
        }

        // 更新安全设置
        _, err := api.UpdateZoneSettings(ctx, zoneID, settings)
        if err != nil {
                return err
        }

        return nil
}

func main() {
        http.HandleFunc("/handle_404", handleRequest)
        logFile, err := os.OpenFile("/var/log/handle404.log", os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0644)
        if err != nil {
                fmt.Println("无法创建日志文件:", err)
                return
        }
        defer logFile.Close()
        log.SetOutput(logFile)
        log.Printf("Server started on :8080\n")
        http.ListenAndServe(":8080", nil)
}